@@ -20,7 +20,6 @@
 #include "internals.h"
 #include "spinlock.h"
 #include "restart.h"
-#include <stackinfo.h>
 
 #ifdef _STACK_GROWS_DOWN
 # define FRAME_LEFT(frame, other) ((char *) frame >= (char *) other)
@@ -208,9 +207,12 @@ void __pthread_perform_cleanup(char *currentframe)
 }
 
 #ifndef SHARED
-/* We need a hook to force the cancelation wrappers to be linked in when
-   static libpthread is used.  */
+/* We need a hook to force the cancelation wrappers and file locking
+   to be linked in when static libpthread is used.  */
 extern const int __pthread_provide_wrappers;
-static const int * const __pthread_require_wrappers =
+static const int *const __pthread_require_wrappers =
   &__pthread_provide_wrappers;
+extern const int __pthread_provide_lockfile;
+static const int *const __pthread_require_lockfile =
+  &__pthread_provide_lockfile;
 #endif
