@@ -1,6 +1,6 @@
 /* Machine-dependent pthreads configuration and inline functions.
-   S390 version.
-   Copyright (C) 1999, 2000, 2001 Free Software Foundation, Inc.
+   64 bit S/390 version.
+   Copyright (C) 2001 Free Software Foundation, Inc.
    Contributed by Martin Schwidefsky (schwidefsky@de.ibm.com).
    This file is part of the GNU C Library.
 
@@ -23,6 +23,11 @@
 # define PT_EI extern inline
 #endif
 
+/* For multiprocessor systems, we want to ensure all memory accesses
+   are completed before we reset a lock.  On other systems, we still
+   need to make sure that the compiler has flushed everything to memory.  */
+#define MEMORY_BARRIER() __asm__ __volatile__ ("bcr 15,0" : : : "memory")
+
 /* Spinlock implementation; required.  */
 PT_EI long int
 testandset (int *spinlock)
@@ -48,16 +53,22 @@ testandset (int *spinlock)
 register char * stack_pointer __asm__ ("15");
 
 /* Return the thread descriptor for the current thread.
-   S/390 registers uses access register 0 as "thread register".  */
+   64 bit S/390 uses access register 0 and 1 as "thread register".  */
 #define THREAD_SELF  ({                                                       \
   register pthread_descr __self;                                              \
-  __asm__ ("ear %0,%%a0" : "=d" (__self) );                                   \
+  __asm__ ("   ear  %0,%%a0\n"                                                \
+           "   sllg %0,%0,32\n"                                               \
+           "   ear  %0,%%a1\n"                                                \
+           : "=d" (__self) );                                                 \
   __self;                                                                     \
 })
 
 /* Initialize the thread-unique value.  */
 #define INIT_THREAD_SELF(descr, nr)  ({                                       \
-  __asm__ ("sar %%a0,%0" : : "d" (descr) );                                   \
+  __asm__ ("   sar  %%a1,%0\n"                                                \
+           "   srlg 0,%0,32\n"                                                \
+           "   sar  %%a0,0\n"                                                 \
+           : : "d" (descr) : "0" );                                           \
 })
 
 /* Access to data in the thread descriptor is easy.  */
@@ -82,21 +93,14 @@ __compare_and_swap(long int *p, long int oldval, long int newval)
         int retval;
 
         __asm__ __volatile__(
-                "  la   1,%1\n"
-                "  lr   0,%2\n"
-                "  cs   0,%3,0(1)\n"
+                "  lgr  0,%2\n"
+                "  csg  0,%3,%1\n"
                 "  ipm  %0\n"
                 "  srl  %0,28\n"
                 "0:"
                 : "=&d" (retval), "+m" (*p)
                 : "d" (oldval) , "d" (newval)
-                : "cc", "0", "1" );
+                : "cc", "0");
         return retval == 0;
 }
 
-
-
-/* Get some notion of the current stack.  Need not be exactly the top
-   of the stack, just something somewhere in the current frame.  */
-#define CURRENT_STACK_FRAME  stack_pointer
-register char * stack_pointer __asm__ ("%r15");
