@@ -63,3 +63,9 @@ register struct _pthread_descr_struct *__thread_self __asm__("%g6");
 #define THREAD_GETMEM_NC(descr, member) __thread_self->member
 #define THREAD_SETMEM(descr, member, value) __thread_self->member = (value)
 #define THREAD_SETMEM_NC(descr, member, value) __thread_self->member = (value)
+
+/* We want the OS to assign stack addresses.  */
+#define FLOATING_STACKS 1
+
+/* Maximum size of the stack if the rlimit is unlimited.  */
+#define ARCH_STACK_MAX_SIZE     8*1024*1024
